%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MatLab script that demonstrates generating tones with an AD9162 using
% an Analog Devices ADS7-V2 pattern generator. The AD9162 is configured
% using ACE UI.
%
% Demonstration Setup:
%   DPG:                ADS7-V2
%   Eval board:         AD9162-FMC-EBZ
%   DPGDownloader Ver:  1.3.77.154(or newer)
%   ACE Ver:            ACE 1.0.2391.879 (or newer)
%   MATLAB Ver:         R2014b
%
%   Output Tone: 400MHz
%   FDATA: 5G
%   FDAC: 5G
%   Interpolation: 1
%   JESD204 Lanes: 8
%
% NOTE: 
%   1. The ACE application UI is used to configure the AD9162.
%   2. No external clock required for this demo.
%
% 06/06/2018
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the AnalogDevices DPG .Net Assembly
NET.addAssembly('AnalogDevices.DPG.Interfaces'); % From GAC

disp('DPG.Interfaces.dll assembly was loaded.');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get an ADS7 DPG device
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[ads7, isDownloadError] = GetAds7(true);   % true = force FPGA config download, false = auto
disp(['Found Device: ', char(ads7.FriendlyName)]);

if (isDownloadError)
    uiwait(msgbox({'FPGA config download ERROR. Exiting.'}, 'MATLAB - AD9162'));
    return; % exit program
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the JESD204 interface for the ADS7.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
jesd204Core = ads7.Jesd204Core;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reset the JESD Core entities
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
jesd204Core.Tx.Reset();
jesd204Core.Clocks.Reset();

frameNum = 0+1;      % first frame index (.net uses 0 based arrays)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Configure the ADS7 for the specific AD9162 dac mode
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
interp = 1;
numLanes = 8;
AD9162Setup(ads7, interp, numLanes);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate vectors for download
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
resolution = 16;
FDAC = 5000000000;    % Data clock frequency in Hz
FDATA = FDAC/interp;  % Rate dac samples are clocked out of DPG in Hz
desired = 400.0e6;    % Output frequency in Hz
samples = 16384*3;    % number of samples in vector. (*3 to support 3/6 lane modes)
cycles = round(desired*samples/FDATA);

if (mod(cycles, 2) == 0)
    %disp('oddifying');
    cycles = cycles + 1;
end

disp(['FDPG: ', num2str(FDATA)]);
disp(['Creating tone with ', num2str(samples), ' samples']);

actual=cycles*FDATA/samples;
x=[0:(2*cycles*pi/samples):2*cycles*pi-(2*cycles*pi/samples)];
Idata = round((2^(resolution-1)-1)*cos(x));  % I data is a cosine wave
Qdata = round((2^(resolution-1)-1)*sin(x));  % Q data is a sine wave

% Fill the vector array for all dacs (i.e. JESD204 M param.)
% DPG arrays are organized as: [DAC][Array of samples]
numDacs = jesd204Core.Tx.M;
if (numDacs > 2)
    dpgArray = [Idata; Qdata; Idata; Qdata];
elseif (numDacs > 1)
    dpgArray = [Idata; Qdata];
else
    dpgArray = Idata;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Download vectors into the ADS7
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Downloading to ADS7');
jesd204Core.Tx.Enable = false;
jesd204Core.Tx.DownloadVector(dpgArray);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Turn on the Start of Buffer output trigger
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
jesd204Core.Tx.TriggerOutMode = AnalogDevices.DPG.Interfaces.TriggerOutMode.Sob;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output a tone
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Retry loop for bringing up link
maxNumTries = 3;
numTries = 0;
while(numTries < maxNumTries)
    
    jesd204Core.Tx.Enable = false;
    
    isLocked = jesd204Core.Clocks.SyncClocks(true);
    disp(['isLocked: ', num2str(isLocked)]);
    
    disp(['Serial Line Rate: ', num2str(jesd204Core.Tx.LineRate)]);
    
    disp('Starting playback');
    jesd204Core.Tx.Enable = true;
    
    uiwait(msgbox({'Please initialize the AD9162 using ACE UI'}, 'MATLAB - AD9162'));
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Verify that the JESD204 link is up.
    % Give up after 'linkUpNumTries' tries.
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    linkUpNumTries = 5;
    linkIsUp = false;
    while(~linkIsUp && linkUpNumTries > 0)
        syncState = char(jesd204Core.Tx.Framers(frameNum).SyncState);
        
        disp(['Link Status: ', syncState]);
        if (strcmp(syncState,'High'))
            linkIsUp = true;
            break;
        else
            linkUpNumTries = linkUpNumTries - 1;
            pause(0.5);
        end
    end
    
    if (linkIsUp)
        break;  % Link is up. Should see tone.
    end
    
    numTries = numTries + 1;
    
    pause(1);
end

if (numTries >= maxNumTries)
    disp(['Not able to generate tone after ', num2str(maxNumTries), ' tries.']);
    return; % exit the program
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Should see tone on spectrum analyzer
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
uiwait(msgbox({'Should see tone on spectrum analyzer'}, 'MATLAB - AD9162'));
